/******************************* airkiss **************************/
//includes
#include "autoconf.h"
#include "platform_stdlib.h"
#include "wifi_conf.h"
#include "osdep_service.h"
#include "airkiss.h"
#include "lwip_netconf.h"
#include "udp.h"
#include <sockets.h>

//macros
#define CHANNEL_SWITCH_TIME 100	//channel switch time,units:ms
#define JOIN_AIRKISS    (uint32_t)(1 << 9)

//extern
extern void wifi_enter_promisc_mode();
extern uint32_t rtw_join_status; 
//gloable 
const airkiss_config_t akconf ={
	(airkiss_memset_fn)&memset,
	(airkiss_memcpy_fn)&memcpy,
	(airkiss_memcmp_fn)&memcmp,
	(airkiss_printf_fn)&printf
};

unsigned char cur_channel = 1;
unsigned char lock_channel = 1;
static _timer timer_handler;
airkiss_context_t *akcontext = NULL;

static int airkiss_finished = 0;
static u8 security_type;
static u8 ak_rx_flag = 0;
u8 airkiss_connection_done = 0;

static char broadcast_addr[6] = {0xff,0xff,0xff,0xff,0xff,0xff};
static u8 gbssid[ETH_ALEN];

//for airkiss send notification
int g_UDPBCServerFd = -1;
struct sockaddr_in g_stUDPBCAddr;

#if CONFIG_LWIP_LAYER
extern struct netif xnetif[NET_IF_NUM]; 
#endif

#define DEVICE_TYPE "gh_390b83082bdd"
#define DEVICE_MAC "00e04c878889" 
#define AIRKISS_LAN_PORT 12476
#define AIRKISS_BIND_PORT 12476


/*
	airkiss active send:
	send udp packets to 255.255.255.255(12476)
	packet is generated by airkiss_lan_pack.
	
	called about every 5s

*/
int airkiss_send_active_lan_discovery_packets(int client_socket_fd, int send_times,u8 *device_mac)
{
#if CONFIG_LWIP_LAYER

	int ret = -1;
	struct sockaddr_in to_addr;
	u8 lan_buf[200]; 
	u16 lan_buf_len = sizeof(lan_buf);
#if 1
	if (send_times != 1) {
		airkiss_printf(AIRKISS_ERROR, "airkiss_send_active_lan_discovery_packets fail!\n"); 
		return -1;
	}

#else
	if (send_times >= 50) {
		airkiss_printf(AIRKISS_ERROR, "send more than 50 times multiples 100ms delay, will exceed 5s!\n"); 
		return -1;
	}
#endif

	ret = airkiss_lan_pack(AIRKISS_LAN_SSDP_NOTIFY_CMD, DEVICE_TYPE, device_mac, 0, 0, lan_buf, &lan_buf_len, &akconf); 
	if (ret != AIRKISS_LAN_PAKE_READY) { 
		airkiss_printf(AIRKISS_ERROR, "airkiss pack lan packet error!"); 
		return -1; 
	} 


	FD_ZERO(&to_addr);
	to_addr.sin_family = AF_INET;
	to_addr.sin_port = htons(AIRKISS_LAN_PORT);
	to_addr.sin_addr.s_addr =inet_addr("255.255.255.255");


	while (send_times-- > 0) {
		airkiss_printf(AIRKISS_INFO, "airkiss_send_active_lan_discovery_packets !\n"); 
		sendto(client_socket_fd, (unsigned char *)lan_buf, lan_buf_len, 0, (struct sockaddr *) &to_addr, sizeof(struct sockaddr));
		vTaskDelay(100);	/* delay 100 ms */
	}


#else
	airkiss_printf(AIRKISS_ERROR, "airkiss_send_active_lan_discovery_packets fail!\n"); 
#endif

	return 0;
}




void airkiss_lan_server_reply(int client_socket_fd, struct sockaddr_in addr, char *pdata, unsigned short len,u8* device_mac) 
{ 
	airkiss_lan_ret_t ret = -1;
	airkiss_lan_ret_t pack_ret; 
	u8 lan_buf[200]; 
	u16 lan_buf_len = sizeof(lan_buf);



	ret = airkiss_lan_recv(pdata, len, &akconf); 

	switch (ret){ 
	case AIRKISS_LAN_SSDP_REQ: 
		addr.sin_port = htons(AIRKISS_LAN_PORT);
		lan_buf_len = sizeof(lan_buf); 
		pack_ret = airkiss_lan_pack(AIRKISS_LAN_SSDP_RESP_CMD,  DEVICE_TYPE, device_mac, 0, 0, lan_buf, &lan_buf_len, &akconf); 
		if (pack_ret != AIRKISS_LAN_PAKE_READY) { 
			airkiss_printf(AIRKISS_ERROR, "Pack lan packet error!"); 
			return; 
		} 
		
		airkiss_printf(AIRKISS_INFO, "airkiss_lan_server_reply !\n"); 
		sendto(client_socket_fd, (unsigned char *)lan_buf, lan_buf_len, 0, (struct sockaddr *) &addr, sizeof(struct sockaddr));
		break; 
	default: 
		airkiss_printf(AIRKISS_ERROR, "Pack is not ssdq req!"); 
	break; 
	} 
} 


int airkiss_lan_server_create(int client_socket_fd,u8 *device_mac)
{
	int server_sock_fd,len;  
	struct sockaddr_in addr;  
	int sock_timeout_val = 1000; /* 1000 ms */
	int timeout_times = 0;
	int addr_len = sizeof(struct sockaddr_in);  
	char buffer[256] = {0};  


	printf("%s\n", __FUNCTION__);

	if((server_sock_fd = socket(AF_INET,SOCK_DGRAM,0)) < 0){  
		airkiss_printf(AIRKISS_ERROR, "create sock error!"); 
		return -1;
	}

	memset(&addr, 0, sizeof(addr));  
	addr.sin_family=AF_INET;  
	addr.sin_port=htons(AIRKISS_BIND_PORT);  
	addr.sin_addr.s_addr=htonl(INADDR_ANY) ;  

	if (bind(server_sock_fd, (struct sockaddr *)&addr, sizeof(addr))<0) {
		airkiss_printf(AIRKISS_ERROR, "bind sock error!"); 
		return -1;
	}

	setsockopt(server_sock_fd, SOL_SOCKET, SO_RCVTIMEO, &sock_timeout_val, sizeof(sock_timeout_val));

	while(1) {  
	  	memset(&buffer, 0, sizeof(buffer));  
		len = recvfrom(server_sock_fd, buffer, sizeof(buffer), 0 , (struct sockaddr *)&addr ,&addr_len);  

		if (-1 == len) {
			/* timeout happens */
			timeout_times++;
			if (5 == timeout_times) {
				timeout_times = 0;
				airkiss_send_active_lan_discovery_packets(server_sock_fd, 1, device_mac);
			}
		} else {
			/* packet received, filter and send back */ 
			airkiss_printf(AIRKISS_INFO, "receive size=%d, from %s\n", len, inet_ntoa( addr.sin_addr));  
			airkiss_lan_server_reply(server_sock_fd, addr, buffer, len, device_mac);
		}
	}

}

/* this is a task */
void airkiss_lan_discovery_start(void* param)
{
	int airkiss_sock_active = -1;
	u8 *dev_mac = (u8 *)param;

#if 0/*use one socket*/
	airkiss_sock_active = socket(PF_INET, SOCK_DGRAM, IP_PROTO_UDP);
	if (airkiss_sock_active == -1) {
		airkiss_printf(AIRKISS_ERROR, "airkiss socket create fail\n");  
		vTaskDelete(NULL);
	}
#endif

	while(!airkiss_connection_done)
		vTaskDelay(600);

	airkiss_printf(AIRKISS_INFO, "airkiss_lan_discovery_start,mac address = %s\n",dev_mac); 
	airkiss_lan_server_create(airkiss_sock_active,dev_mac);
	/*since lan discovery should be send always,so we will never reach this*/
	vTaskDelete(NULL);
}

void  airkiss_landiscovery_task(void* param)
{
	if(xTaskCreate(airkiss_lan_discovery_start, "airkiss_lan_discovery", 512, param, tskIDLE_PRIORITY + 3, NULL) != pdPASS)
		airkiss_printf(AIRKISS_INFO, "\n xTaskCreate(airkiss_lan_discovery) failed");
}

int airkiss_createudpbroadcast(int udp_port)
{

	int ret = 0;
	int udpbufsize = 2;
	struct sockaddr_in g_stUDPBCServerAddr;

	if (g_UDPBCServerFd == -1) {
		
		g_UDPBCServerFd = socket(AF_INET, SOCK_DGRAM, 0);
		
		if (g_UDPBCServerFd < 0) {
			printf("UDP BC socket create error,g_UDPBCServerFd = %d\r\n",g_UDPBCServerFd);
			g_UDPBCServerFd = -1;
			ret = -1;
			return ret;
		}

		g_stUDPBCAddr.sin_family = AF_INET;
		g_stUDPBCAddr.sin_port = htons(udp_port);
		g_stUDPBCAddr.sin_addr.s_addr = htonl(0xffffffff);

		g_stUDPBCServerAddr.sin_family = AF_INET;
		g_stUDPBCServerAddr.sin_port = htons(udp_port);
		g_stUDPBCServerAddr.sin_addr.s_addr = htonl(INADDR_ANY);

		if (setsockopt(g_UDPBCServerFd, SOL_SOCKET, SO_BROADCAST, &udpbufsize,sizeof(int))!= 0) {
			ret = -1;
			printf("UDP BC Server setsockopt error\n");
			return ret;
		}
		if (bind(g_UDPBCServerFd, (struct sockaddr *)&g_stUDPBCServerAddr, sizeof(g_stUDPBCServerAddr))!= 0) {
			printf("UDP BC Server bind error\n");
			close(g_UDPBCServerFd);
			g_UDPBCServerFd = -1;
			ret = -1;
			return ret;
		}

	}
	printf("UDP BC Server socketid:%d on port:%d\n", g_UDPBCServerFd,udp_port);
	return ret;
}

int airkiss_send_notification(u8 random)
{
	int i = 0, ret = 0;
	if(airkiss_createudpbroadcast(10000) == -1){
		ret = -1;
		printf("create udp socket fail!\r\n");
		return ret;
	}
		
	for (i = 0; i <= 20; i++) {
		if (g_UDPBCServerFd > -1) {
			ret = sendto(g_UDPBCServerFd, (char*)&random, 1, 0 ,(struct sockaddr*)&g_stUDPBCAddr, sizeof(g_stUDPBCAddr));			
		}
	}
	close(g_UDPBCServerFd);
	g_UDPBCServerFd = 0;
	return ret;
}


void airkiss_deinit_content()
{
	
	int i = 0;
	
	//free airkiss context at last
	if(akcontext){
		rtw_free(akcontext);
		akcontext = NULL;
	}
	
	rtw_del_timer(&timer_handler);
	
}

int airkiss_get_macaddress( u8* akmac )
{
	u8* tmp;
	u8 mac[6]={0};
	
	if(akmac == NULL){
		return -1;
	}
	
	/*get mac address*/
	tmp= LwIP_GetMAC(&xnetif[0]);
	
	mac[0] = tmp[0];
	mac[1] = tmp[1];
	mac[2] = tmp[2];
	mac[3] = tmp[3];
	mac[4] = tmp[4];
	mac[5] = tmp[5];

	sprintf((char *)akmac,"%02X%02X%02X%02X%02X%02X",mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	return 0;
}
int airkiss_finish(unsigned char security_type,  rtw_network_info_t * wifi,airkiss_context_t *context)
{
	int ret = 0;
	int retry = 3;
	unsigned char pscan_config = 1;
	airkiss_result_t result;
	rtw_security_t security_mode;
	ret = airkiss_get_result(context, &result);
	
	if (ret == 0) {
		if(wifi){
  		/*memcpy profile information to 3rd party variables*/
  	 	wifi->password_len = result.pwd_length;
	 	wifi->ssid.len = result.ssid_length;
	 	rtw_memcpy(wifi->ssid.val, result.ssid, result.ssid_length);
	 	rtw_memcpy(wifi->password, result.pwd, result.pwd_length);
	 	rtw_memcpy(wifi->bssid.octet, gbssid, ETH_ALEN);
  		}
		airkiss_printf(AIRKISS_INFO,"\r\n airkiss get result ok,ssid = %s, pwd = %s,ssid length = %d,pwd length = %d,random = 0x%02x",
			       result.ssid, result.pwd, result.ssid_length,result.pwd_length,result.random);
	}
	else{
		airkiss_printf(AIRKISS_ERROR,"\r\n airkiss result not get!");
		return -1;
	}
	//ap security type
	switch(security_type){
	case RTW_ENCRYPTION_OPEN:
		security_mode = RTW_SECURITY_OPEN;
		break;
	case RTW_ENCRYPTION_WEP40:
	case RTW_ENCRYPTION_WEP104:
		security_mode = RTW_SECURITY_WEP_PSK;
		break;
	case RTW_ENCRYPTION_WPA_TKIP:
	case RTW_ENCRYPTION_WPA_AES:
	case RTW_ENCRYPTION_WPA2_TKIP:
	case RTW_ENCRYPTION_WPA2_AES:
	case RTW_ENCRYPTION_WPA2_MIXED:
		security_mode = RTW_SECURITY_WPA2_AES_PSK;
		break;
	case RTW_ENCRYPTION_UNKNOWN:
	case RTW_ENCRYPTION_UNDEF:
	default:
		airkiss_printf(AIRKISS_ERROR,"\r\n unknow security mode,connect fail!");
	}
	
	if(wifi){
  		wifi->security_type = security_mode;
  	}
	while(retry){
		if(wifi_set_pscan_chan(&lock_channel, &pscan_config, 1) < 0){
			airkiss_printf(AIRKISS_ERROR,"\n\rERROR: wifi set partial scan channel fail");
			break;
		}
		airkiss_printf(AIRKISS_INFO,"wifi_connect\n");
		rtw_join_status = 0; // clear status to enable wifi_connect
		ret = wifi_connect(result.ssid, security_mode, 
				   result.pwd, result.ssid_length, 
				   result.pwd_length, 
				   0,NULL);
		if(ret == 0){
			airkiss_printf(AIRKISS_INFO,"\n\r Connect ok!");
#if CONFIG_LWIP_LAYER
			/* If not rise priority, LwIP DHCP may timeout */
//			vTaskPrioritySet(NULL, tskIDLE_PRIORITY + 3);	
			/* Start DHCP Client */
			ret = LwIP_DHCP(0, DHCP_START);
//			vTaskPrioritySet(NULL, tskIDLE_PRIORITY + 1);	
#endif
			break;
		}
		retry--;
	}

	airkiss_deinit_content();

	//airkiss send notification
	if(airkiss_send_notification(result.random) == -1)
		airkiss_printf(AIRKISS_ERROR, "airkiss send notifacation fail\r\n");
	
#if 1
	  /*set airkiss_connection_done*/
//	if(wifi) {
		airkiss_connection_done = 1;
//	}
#endif  

#if CONFIG_INIC_CMD_RSP
	inic_c2h_wifi_info("ATWX", ret);
#endif

	return 0;
	
}
void timer_handler_func(void *FunctionContext)
{
	
	if(ak_rx_flag){
		rtw_set_timer(&timer_handler, CHANNEL_SWITCH_TIME - 50);
	} else {
		if(cur_channel >= 13)
			cur_channel = 1;
		else
			cur_channel ++;
		wifi_set_channel(cur_channel);
		/* inform airkiss we have changed channel */
		airkiss_change_channel(akcontext);
		rtw_set_timer(&timer_handler, CHANNEL_SWITCH_TIME);
	}
	
	// wifi_set_channel(8);
	airkiss_printf(AIRKISS_INFO,"\r\n wifi switch channel to %d",cur_channel);
}

static int rtl_promisc_is_ap(unsigned short i_fc)
{
	if ((i_fc & DS_FRAME_MASK) == GET_AP_FRAME)
		return 1;
	else
		return 0;
}

static int rtl_promisc_is_sta(unsigned short i_fc)
{
	if ((i_fc & DS_FRAME_MASK) == GET_STA_FRAME)
		return 1;
	else
		return 0;
}



static void rtl_frame_recv_handle(char *buf, int len) {
	
	int ret = 0;
	int fixed_channel;
	char scanned_ssid[50] = {0};
	char *current_bssid = NULL;
	int scanned_ssid_len = 0;
	
	//set this flag prevent airkiss_recv interruptted by timer,since timer has higher priority
	ak_rx_flag = 1;
	if (airkiss_finished) {
		ak_rx_flag = 0;
		return;
	}
	
	ret = airkiss_recv(akcontext, buf, len);
	if(ret == AIRKISS_STATUS_CHANNEL_LOCKED){  
		lock_channel = cur_channel;
		rtw_cancel_timer(&timer_handler);
		airkiss_printf(AIRKISS_INFO,"\r\n AirKiss locked to channel[%d]\n",lock_channel);
		
		current_bssid = buf + 4 + ETH_ALEN;
		
		/*printf("current bssid = %02x:%02x:%02x:%02x:%02x:%02x\n", 
		current_bssid[0],current_bssid[1],current_bssid[2],
		current_bssid[3],current_bssid[4],current_bssid[5]);
		*/
		fixed_channel = promisc_get_fixed_channel(current_bssid, scanned_ssid, &scanned_ssid_len);
		if (fixed_channel != 0) {
			airkiss_printf(AIRKISS_INFO,"AirKiss force fixed to channel[%d]\r\n",fixed_channel);
			airkiss_printf(AIRKISS_INFO,"AirKiss ssid scanned[%s]\r\n",scanned_ssid);
			wifi_set_channel(fixed_channel);
		}
		
	}
	else if(ret == AIRKISS_STATUS_COMPLETE){
		airkiss_finished = 1;
		airkiss_printf(AIRKISS_INFO,"airkiss finished\r\n");
	} 
	//release flag
	ak_rx_flag = 0;
	
}



void wifi_promisc_rx(unsigned char* buf, unsigned int len, void* user_data)
{
	unsigned char ret = 0;
	//	int index = 0;
	ieee80211_frame_info_t *promisc_info;
	promisc_info = user_data;
	
	if (airkiss_finished)
		return;
	
	security_type = promisc_info->encrypt;
	if (rtl_promisc_is_ap(promisc_info->i_fc)) {
		if (memcmp(promisc_info->i_addr1, broadcast_addr, ETH_ALEN) != 0)
			return;
	}
	
	if (rtl_promisc_is_sta(promisc_info->i_fc)) {
		if (memcmp(promisc_info->i_addr3, broadcast_addr, ETH_ALEN) != 0)
			return;
	}
	
	
	rtl_frame_recv_handle(buf, len);
	
}

void airkiss_process(void *param)
{
	rtw_network_info_t *wifi = NULL;
	wifi = (rtw_network_info_t *)param;
	u32 start_time = xTaskGetTickCount();
	
	while(!airkiss_finished ){
		vTaskDelay(15);
	}
	/*quite promisc mode here*/
	wifi_set_promisc(RTW_PROMISC_DISABLE,NULL,0);
	
	if(!airkiss_finished)
		airkiss_printf(AIRKISS_ERROR,"\n\rAirkiss timeout!");
	else {
		airkiss_printf(AIRKISS_INFO,"\r\nAirkiss complete!");
		
		airkiss_finish(security_type, wifi,akcontext);

	}
	vTaskDelete(NULL);
}

int airkiss_init_content(airkiss_context_t **content)
{
	
	int ret = 0;
	
	//free airkiss context if still exists
	if(*content){
		rtw_free(*content);
		*content = NULL;
	}
	
	//init airkiss context
	*content = (airkiss_context_t *)rtw_malloc(sizeof(airkiss_context_t));
	if(!(*content)){
		airkiss_printf(AIRKISS_ERROR,"\n\rAlloc airkiss context failed!");
		ret = -1;
		return ret;
	}
	
	//init airkiss
	ret = airkiss_init(*content,&akconf);
	if(ret < 0){
		airkiss_printf(AIRKISS_ERROR,"Airkiss init failed!\r\n");
		return ret;
	}
	
#if AIRKISS_ENABLE_CRYPT
	airkiss_set_key(*content, key, strlen(key));
#endif
	
	return 0;
}

int airkiss_start(rtw_network_info_t *wifi)
{
	char ret = 0;
	u8 akmac[32+1];
	airkiss_finished = 0;
	rtw_join_status |= JOIN_AIRKISS;
	
#if AIRKISS_ENABLE_CRYPT
	const char *key = "wechatiothardwav";
#endif
	if(xTaskCreate(airkiss_process, (char const *)"AirKiss", 1512, wifi, tskIDLE_PRIORITY + 3, NULL) != pdPASS)
		airkiss_printf(AIRKISS_ERROR,"\n\r%s xTaskCreate failed", __FUNCTION__);
	/*get mac address for airkiss2.0 lan discovery*/
	airkiss_get_macaddress(akmac);
	airkiss_landiscovery_task(akmac);
	//get airkiss version
	airkiss_printf(AIRKISS_INFO,"\n\rAirkiss version %s",airkiss_version());
	if (airkiss_init_content(&akcontext) < 0)
		airkiss_printf(AIRKISS_ERROR,"airkiss init fail!\n");
	
	//set wifi to station mode,enable promisc mode and timer to change channel
	wifi_enter_promisc_mode();
	
	/* enable all 802.11 packets*/
	wifi_set_promisc(RTW_PROMISC_ENABLE_2, wifi_promisc_rx, 1);
	
	//init timer handler,and set timer hanler funcion
	rtw_init_timer(&timer_handler, NULL, &timer_handler_func, NULL, "airkiss_timer");
	//set channel
	wifi_set_channel(cur_channel);
	//set timer to CHANNEL_SWITCH_TIME(ms)	
	rtw_set_timer(&timer_handler, CHANNEL_SWITCH_TIME);
	
	return ret;
}
/*********************************************************/
